//
// 2008.08 Kyoung Shin Park 
//
//#include <TCHAR.h>
#include <stdio.h>
#include <windows.h>
#include "SharedMemory.h"

CSharedMemory::CSharedMemory(string name)
{
	m_Name = name;
	m_MemoryName = "SHARED_MEMORY" + m_Name;
	m_MutexName = "SHARED_MEMORY_MUTEX" + m_Name;
}

void CSharedMemory::createMutex()
{
	//m_KeyValue = 0x3829; // just an arbitrary number
	m_hKeyMutex = CreateMutex(NULL, FALSE, m_MutexName.c_str());
	if (m_hKeyMutex == NULL) {
		printf("CreateMutex(): error: %d.\n", GetLastError());
	}
	else {
		if (GetLastError() == ERROR_ALREADY_EXISTS)
			printf("CreateMutex(): opened existing %s mutex.\n", m_MutexName.c_str());
		else
			printf("CreateMutex(): new %s mutex successfully created.\n", m_MutexName.c_str());
	}
}

void CSharedMemory::openMutex()
{
	m_hKeyMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, m_MutexName.c_str());
	if (m_hKeyMutex == NULL) {
		printf("OpenMutex(): error: %d.\n", GetLastError());
	}
	else {
		printf("OpenMutex(): %s mutex successfully opened.\n", m_MutexName.c_str());
	}
}

void CSharedMemory::closeMutex()
{
	if (m_hKeyMutex)
		CloseHandle(m_hKeyMutex);
}

void CSharedMemory::lockMutex()
{
	WaitForSingleObject(m_hKeyMutex, INFINITE);
}

void CSharedMemory::unlockMutex()
{
	ReleaseMutex(m_hKeyMutex);
}

HANDLE CSharedMemory::getShm(key_t key, size_t size, int shmflg)
{
	//TCHAR memoryMapName[256];
	DWORD flag = 0;

	if (key == IPC_PRIVATE || (shmflg & IPC_CREAT)) {
		//WaitForSingleObject(m_hKeyMutex, INFINITE);
		//_stprintf(memoryMapName, "YCSHAREDMEMORY%d", KeyValue);
		//KeyValue++; 
		//ReleaseMutex(m_hKeyMutex);

		if (shmflg & S_IWUSR) 
			flag = PAGE_READWRITE;
		else if (shmflg & S_IRUSR)
			flag = PAGE_READONLY;
		m_hMemory = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, flag, 0, size, m_MemoryName.c_str());
		if (m_hMemory == NULL) {
			int error = GetLastError();
			if (error == ERROR_DISK_FULL) {
				//errno = ENOMEM;
				return NULL;
			}
		}
		else if ((shmflg & IPC_EXCL) && GetLastError() == ERROR_ALREADY_EXISTS) {
			//errno = EEXIST;
			return NULL;
		}
		return m_hMemory;
	}
	else {
		if (shmflg & S_IWUSR) 
			flag |= FILE_MAP_WRITE;
		if (shmflg & S_IRUSR)
			flag |= FILE_MAP_READ;
		m_hMemory = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, m_MemoryName.c_str());
		if (m_hMemory == NULL) {
			//errno = ENOENT;
			return NULL;
		}
		return m_hMemory;
	}
	return NULL;

}

int CSharedMemory::cleanShm(HANDLE shmid, int cmd, struct shmid_ds *buf)
{
	if (cmd == IPC_RMID) {
		if (shmid) {
			CloseHandle(shmid);
			return 0;
		}
		return -1;
	}
	return -1;
}

void * CSharedMemory::attachShm(HANDLE shmid, const void *shmaddr, int shmflg)
{
	if (shmid) {
		void *ptr = MapViewOfFile(shmid, FILE_MAP_ALL_ACCESS, 0, 0, 0);
		if (ptr) return ptr;
	}
	return NULL;
}

int CSharedMemory::detachShm(const void *shmaddr)
{
	if (UnmapViewOfFile(shmaddr)) return 0;
	return -1;
}


